package edu.tum.motras.algorithms;

import static edu.tum.motras.algorithms.util.Util.getMustTransitions;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.google.common.collect.ImmutableSet;

import edu.tum.xmts.State;
import edu.tum.xmts.Transition;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.TransitionSystem.Type;

public class CompositionMTS extends AbstractComposition {

	public CompositionMTS(String string) {
		super(string);
	}

	public CompositionMTS() {
		super();
	}

	@Override
	public Set<Type> getSupportedTypes() {
		return ImmutableSet.of(Type.MTS);
	}

	@Override
	public String toString() {
		return "Composition (MTS)";
	}

	@Override
	protected void setup(TransitionSystem system) {
		system.setId("composition");
	}

	@Override
	protected void setObligation(State state, State leftState, State rightState,
			Map<Transition, Transition> translationMap) {
		List<Transition> newMustTransitions = new LinkedList<>();

		for (Transition transition1 : getMustTransitions(leftState)) {
			Transition transition = translationMap.get(transition1);

			if (transition != null) {
				newMustTransitions.add(transition);
			}
		}

		for (Transition transition2 : getMustTransitions(rightState)) {
			Transition transition = translationMap.get(transition2);

			if (transition != null) {
				newMustTransitions.add(transition);
			}
		}

		state.setMustObligation(newMustTransitions);
	}
}
