package edu.tum.motras.algorithms;

import static edu.tum.motras.algorithms.util.Util.checkArguments;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Set;

import com.google.common.collect.ImmutableSet;

import edu.tum.motras.algorithms.util.ExternalTool;
import edu.tum.motras.algorithms.util.MoTraSWriter;
import edu.tum.xmts.State;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.TransitionSystem.Type;

public class LTLChecker implements Algorithm<Boolean> {

	private class ltl2dstar extends ExternalTool<String, Void> {

		protected ltl2dstar(File rabin) throws IOException {
			super("ltl2dstar", "--ltl2nba=spin:ltl2ba", "-", rabin.getAbsolutePath());
		}

		@Override
		protected Void readOutput(int exitValue, String stdout) {
			return null;
		}

		@Override
		protected void writeInput(Writer stdin, String input) throws IOException {
			try (BufferedWriter bufferedWriter = new BufferedWriter(stdin)) {
				bufferedWriter.write(input);
			}
		}
	}

	private class MoTraS extends ExternalTool<Void, TransitionSystem> {

		protected MoTraS(String state, File rabin, File dmts) throws IOException {
			super("motras", "-s" + state, "-r" + rabin.getAbsolutePath(), dmts.getAbsolutePath());
		}

		@Override
		protected TransitionSystem readOutput(int exitValue, String stdout) {
			if (stdout.contains("NO")) {
				return null;
			}

			return new TransitionSystem("Implementation");
		}

		@Override
		protected void writeInput(Writer stdin, Void input) {
			return;
		}
	}

	private String ltlFormula;

	public LTLChecker() {
		ltlFormula = "";
	}

	public LTLChecker(String string) {
		setConfiguration(string);
	}

	@Override
	public boolean isConfigurable() {
		return true;
	}

	@Override
	public void setConfiguration(String conf) {
		ltlFormula = conf.toUpperCase().trim();
	}

	@Override
	public int getArity() {
		return 1;
	}

	@Override
	public Set<Type> getSupportedTypes() {
		return ImmutableSet.of(Type.MTS, Type.DMTS);
	}

	@Override
	public Boolean apply(State... states) throws IOException {
		checkArguments(this, states);

		File rabin = File.createTempFile("motras-algorithms", ".rabin");
		File dmts = File.createTempFile("motras-algorithms", ".dmts");

		rabin.createNewFile();
		dmts.createNewFile();
		rabin.deleteOnExit();
		dmts.deleteOnExit();

		/* Write Rabin */
		ltl2dstar ltl2dstarTool = new ltl2dstar(rabin);
		ltl2dstarTool.run(ltlFormula);

		/* Write Automaton */
		MoTraSWriter writer = new MoTraSWriter();

		try (FileWriter fileWriter = new FileWriter(dmts)) {
			writer.write(states[0].getContext(), fileWriter);
			fileWriter.close();
		}

		MoTraS moTraS = new MoTraS(states[0].getId().toUpperCase(), rabin, dmts);

		return moTraS.run(null) != null ? Boolean.TRUE : Boolean.FALSE;
	}

	@Override
	public String toString() {
		return "LTLChecker";
	}
}
