package edu.tum.motras.algorithms;

import static edu.tum.motras.algorithms.util.Util.checkArguments;

import java.util.Set;

import com.google.common.collect.ImmutableSet;

import edu.tum.xmts.State;
import edu.tum.xmts.Transition;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.TransitionSystem.Type;

public class MaxImplementation extends WaitingQueueAlgorithm<State> implements Algorithm<State> {

	@Override
	public State apply(State... input) throws IllegalArgumentException {
		checkArguments(this, input);

		run(input[0]);
		return transitionSystem.getInitialState();
	}

	@Override
	public int getArity() {
		return 1;
	}

	@Override
	public Set<Type> getSupportedTypes() {
		return ImmutableSet.of(Type.MTS, Type.DMTS);
	}

	@Override
	public String toString() {
		return "Max Implementation";
	}

	private State getState(State state) {
		State newState = stateMapper.map(state);
		queue(state);
		return newState;
	}

	@Override
	protected void process(State element) {
		State state = stateMapper.map(element);

		for (Transition transition : element.getTransitions()) {
			State nextState = getState(transition.getState());
			state.createTransition(actionMapper.map(transition.getAction()), nextState);
		}

		state.setAllObligation();
	}

	@Override
	protected void setup(TransitionSystem transitionSystem) {
		transitionSystem.setId("MaxImplementation");
	}

	@Override
	public boolean isConfigurable() {
		return false;
	}

	@Override
	public void setConfiguration(String conf) {
		throw new UnsupportedOperationException("This algorithm cannot be configured");
	}
}
