package edu.tum.motras.algorithms;

import static edu.tum.motras.algorithms.util.Util.checkArguments;

import java.util.Collection;
import java.util.Set;

import com.google.common.collect.Collections2;
import com.google.common.collect.ImmutableSet;

import edu.tum.motras.algorithms.util.StatePair;
import edu.tum.motras.algorithms.util.Util;
import edu.tum.motras.algorithms.util.Util.RelationConstruction;
import edu.tum.xmts.State;
import edu.tum.xmts.Transition;
import edu.tum.xmts.TransitionSystem.Type;

public class ModalRefinementRelationFixedPoint extends FixedPointAlgorithm<StatePair> implements
		Algorithm<Set<StatePair>> {

	private RelationConstruction strategy = RelationConstruction.REACHABLE;

	@Override
	public boolean isConfigurable() {
		return true;
	}

	@Override
	public void setConfiguration(String conf) {
		this.strategy = RelationConstruction.valueOf(conf);
	}

	@Override
	public Set<StatePair> apply(State... input) {
		checkArguments(this, input);

		Set<StatePair> relation = null;

		switch (strategy) {
		case CARTESIAN:
			relation = Util.getCartesianProduct(input[0], input[1]);
			break;

		case REACHABLE:
			relation = Util.getReachables(input[0], input[1]);
			break;
		}

		run(relation);

		return relation;
	}

	@Override
	public int getArity() {
		return 2;
	}

	@Override
	public Set<Type> getSupportedTypes() {
		return ImmutableSet.of(Type.MTS);
	}

	@Override
	public String toString() {
		return "Modal Refinement Relation (Fixed Point)";
	}

	@Override
	protected boolean check(StatePair element) {
		State leftState = element.get(1);
		State rightState = element.get(2);

		/* May Transitions */
		for (Transition leftTransition : leftState.getTransitions()) {
			boolean matched = false;

			for (Transition rightTransition : rightState.getTransitions(leftTransition.getAction().getId())) {
				StatePair nextPair = new StatePair(leftTransition.getState(), rightTransition.getState());

				if (relation.contains(nextPair)) {
					matched = true;
					break;
				}
			}

			if (!matched) {
				return false;
			}
		}

		/* Must Transitions */
		for (Transition rightTransition : Util.getMustTransitions(rightState)) {
			Collection<Transition> leftMustTransitions = Collections2.filter(Util.getMustTransitions(leftState),
					new Util.ActionFilter(rightTransition.getAction().getId()));

			boolean matched = false;

			for (Transition leftTransition : leftMustTransitions) {
				StatePair nextPair = new StatePair(leftTransition.getState(), rightTransition.getState());

				if (relation.contains(nextPair)) {
					matched = true;
					break;
				}
			}

			if (!matched) {
				return false;
			}
		}

		return true;
	}
}
