package edu.tum.motras.algorithms.util;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import edu.tum.xmts.GenericRecord;
import edu.tum.xmts.State;
import edu.tum.xmts.Transition;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.utils.TransitionSystemWriter;
import edu.tum.xmts.utils.XMTSReader;
import edu.tum.xmts.utils.XMTSReader.MalformedInputException;

public class MIOWriter implements TransitionSystemWriter {

	public static void main(String[] args) throws MalformedInputException {
		String xmts1 = "mts test1[action a action b action c " + "state s1 [obligation all  a->s2]"
				+ "state s2 [obligation true b->s1 c->s1]]";

		XMTSReader reader = new XMTSReader();
		TransitionSystemWriter writer = new MIOWriter();
		System.out.print(writer.write(reader.read(xmts1)));
	}

	@Override
	public String write(TransitionSystem transitionSystem) {
		StringWriter stringWriter = new StringWriter();

		try {
			write(transitionSystem, stringWriter);
		} catch (IOException e) {
			e.printStackTrace();
		}

		return stringWriter.toString();
	}

	@Override
	public void write(TransitionSystem ts, Writer w) throws IOException {
		w.write("mio T" + ts.getId() + " {\n");

		/* Write Actions */
		w.write("    inputs ");
		write(ts.actions.getElements(), w, "a_");
		w.write("\n");

		/* Write States */
		w.write("    states ");
		write(ts.states.getElements(), w, "s_");
		w.write("\n");

		/* Write Initial State */
		State initialState = ts.getInitialState();

		if (initialState == null) {
			w.write("    start s_" + ts.states.getElements().get(0).getId() + "\n");
		} else {
			w.write("    start s_" + initialState.getId() + "\n");
		}

		List<Transition> may = new LinkedList<>();
		List<Transition> must = new LinkedList<>();

		for (State state : ts.states.getElements()) {
			may.addAll(state.getTransitions());
			must.addAll(Util.getMustTransitions(state));
		}

		/* Write May Transitions */
		w.write("    mayTransitions\n");
		write(may, w);

		/* Write May Transitions */
		w.write("    mustTransitions\n");
		write(must, w);

		w.write("}");
	}

	private static void write(List<Transition> transitions, Writer w) throws IOException {
		Iterator<Transition> iter = transitions.iterator();

		while (iter.hasNext()) {
			Transition current = iter.next();
			w.write("        s_" + current.getContext().getId() + " -> s_" + current.getState().getId() + "[a_"
					+ current.getAction().getId() + "]");

			if (iter.hasNext()) {
				w.write(",\n");
			}
		}

		w.write("\n");
	}

	private static void write(List<? extends GenericRecord<?>> records, Writer w, String prefix) throws IOException {
		Iterator<? extends GenericRecord<?>> iter = records.iterator();

		while (iter.hasNext()) {
			GenericRecord<?> current = iter.next();
			w.write(prefix + current.getId());

			if (iter.hasNext()) {
				w.write(", ");
			}
		}
	}

}
