package edu.tum.motras.algorithms.util;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import com.google.common.collect.ImmutableSet;

import edu.tum.xmts.Action;
import edu.tum.xmts.State;
import edu.tum.xmts.Transition;

public class StateSet {

	private final Set<State> states;

	public StateSet(Collection<State> states) {
		this.states = ImmutableSet.copyOf(states);
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}

		StateSet other = (StateSet) obj;
		return (states.equals(other.states));
	}

	public Set<Action> getActions() {
		Set<Action> set = new HashSet<>();

		for (State state : states) {
			for (Action action : state.getActions()) {
				set.add(action);
			}
		}

		return set;
	}

	public Set<State> getSuccessors(Action action) {
		Set<State> set = new HashSet<>();

		for (State state : states) {
			for (Transition transition : state.getTransitions(action)) {
				set.add(transition.getState());
			}
		}

		return set;
	}

	public Set<State> getStates() {
		return states;
	}

	@Override
	public int hashCode() {
		return states.hashCode();
	}

	public boolean isEqual(Collection<State> states) {
		StateSet other = new StateSet(states);
		return this.equals(other);
	}

	@Override
	public String toString() {
		return "StateSet [states=" + states + "]";
	}
}
