package edu.tum.xmts;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkState;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

@NonNullByDefault
public class Parameter extends GenericRecord<TransitionSystem> {

	/* XMTS Properties */

	public static final String PROP_INVESTMENT_COST = "investment_cost";

	private @Nullable Integer investmentCost;

	Parameter(TransitionSystem context, String id) {
		super(context, id);
	}

	public void clearInvestmentCost() {
		setInvestmentCost(null);
	}

	public @Nullable Integer getInvestmentCost() {
		return investmentCost;
	}

	public void setInvestmentCost(@Nullable Integer investmentCost) {
		checkState(isSupportedProperty(PROP_INVESTMENT_COST));
		checkArgument(investmentCost == null || investmentCost.intValue() >= 0,
				"InvestmentCost is expected nonnegative or null", investmentCost);

		Integer oldInvestmentCost = this.investmentCost;
		this.investmentCost = investmentCost;
		propertyChangeSupport.firePropertyChange(PROP_INVESTMENT_COST, oldInvestmentCost, investmentCost);
	}

	@Override
	public boolean isSupportedProperty(String propName) {
		if (super.isSupportedProperty(propName)) {
			return true;
		}

		// Only mtsd+dps allows additional properties for actions and parameters
		if (getContext().getType() != TransitionSystem.Type.MTSD_DPS) {
			return false;
		}

		return propName.equals(PROP_INVESTMENT_COST);
	}
}
