package edu.tum.motras.algorithms.junit;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.File;

import org.junit.Before;
import org.junit.Test;

import com.google.common.collect.ImmutableSet;

import edu.tum.motras.algorithms.ModalRefinementQSat;
import edu.tum.xmts.State;
import edu.tum.xmts.TransitionSystem.Type;
import edu.tum.xmts.utils.XMTSReader;

public class ModalRefinementQSatTest {

	private ModalRefinementQSat modalRefinementQSat;

	@Before
	public void setUp() throws Exception {
		modalRefinementQSat = new ModalRefinementQSat();
	}

	@Test
	public void testApply() throws Exception {
		XMTSReader reader = new XMTSReader();

		State simple1 = reader.read(new File("../xmts-specification/examples/traffic/simple1.xmts")).getInitialState();
		State simple2 = reader.read(new File("../xmts-specification/examples/traffic/simple2.xmts")).getInitialState();
		State param1 = reader.read(new File("../xmts-specification/examples/traffic/param1.xmts")).getInitialState();
		State param2 = reader.read(new File("../xmts-specification/examples/traffic/param2.xmts")).getInitialState();

		assertTrue(modalRefinementQSat.apply(simple1, simple1));
		assertFalse(modalRefinementQSat.apply(simple1, simple2));
		assertTrue(modalRefinementQSat.apply(simple1, param1));
		assertTrue(modalRefinementQSat.apply(simple1, param2));

		assertFalse(modalRefinementQSat.apply(simple2, simple1));
		assertTrue(modalRefinementQSat.apply(simple2, simple2));
		assertTrue(modalRefinementQSat.apply(simple2, param1));
		assertTrue(modalRefinementQSat.apply(simple2, param2));

		assertFalse(modalRefinementQSat.apply(param1, simple1));
		assertFalse(modalRefinementQSat.apply(param1, simple2));
		assertTrue(modalRefinementQSat.apply(param1, param1));
		assertTrue(modalRefinementQSat.apply(param1, param2));

		assertFalse(modalRefinementQSat.apply(param2, simple1));
		assertFalse(modalRefinementQSat.apply(param2, simple2));
		assertFalse(modalRefinementQSat.apply(param2, param1));
		assertTrue(modalRefinementQSat.apply(param2, param2));
	}

	@Test
	public void testGetArity() {
		AlgorithmTestUtil.testGetArity(modalRefinementQSat, 2);
	}

	@Test
	public void testGetSupportedTypes() {
		AlgorithmTestUtil.testGetSupportedTypes(modalRefinementQSat,
				ImmutableSet.of(Type.MTS, Type.DMTS, Type.BMTS, Type.PMTS));
	}

}
